#include "lapacknames.inc"
      SUBROUTINE CGESC2( N, A, LDA, RHS, IPIV, JPIV, SCALE )
      USE LA_CONSTANTS
      USE LA_AUXILIARY, ONLY: LA_LASWP
      USE LA_BLAS1, ONLY: LA_IAMAX, LA_SCAL
*
*  -- LAPACK auxiliary routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     June 30, 1999
*     06-06-02:  LAPACK 3E version (eca)
*
*     .. Scalar Arguments ..
      INTEGER            LDA, N
      REAL(WP)           SCALE
*     ..
*     .. Array Arguments ..
      INTEGER            IPIV( * ), JPIV( * )
      COMPLEX(WP)        A( LDA, * ), RHS( * )
*     ..
*
*  Purpose
*  =======
*
*  CGESC2 solves a system of linear equations
*
*            A * X = scale* RHS
*
*  with a general N-by-N matrix A using the LU factorization with
*  complete pivoting computed by CGETC2.
*
*  Arguments
*  =========
*
*  N       (input) INTEGER
*          The number of columns of the matrix A.
*
*  A       (input) COMPLEX array, dimension (LDA, N)
*          On entry, the  LU part of the factorization of the n-by-n
*          matrix A computed by CGETC2:  A = P * L * U * Q
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.  LDA >= max(1, N).
*
*  RHS     (input/output) COMPLEX array, dimension (N)
*          On entry, the right hand side vector b.
*          On exit, the solution vector X.
*
*  IPIV    (input) INTEGER array, dimension (N)
*          The pivot indices; for 1 <= i <= N, row i of the
*          matrix has been interchanged with row IPIV(i).
*
*  JPIV    (input) INTEGER array, dimension (N)
*          The pivot indices; for 1 <= j <= N, column j of the
*          matrix has been interchanged with column JPIV(j).
*
*  SCALE   (output) REAL
*          On exit, SCALE contains the scale factor. SCALE is chosen
*          0 <= SCALE <= 1 to prevent overflow in the solution.
*
*  Further Details
*  ===============
*
*  Based on contributions by
*     Bo Kagstrom and Peter Poromaa, Department of Computing Science,
*     Umea University, S-901 87 Umea, Sweden.
*
*  =====================================================================
*
*     .. Local Scalars ..
      INTEGER            I, J
      REAL(WP)           TEMP
      COMPLEX(WP)        CTEMP
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX
*     ..
*     .. Executable Statements ..
*
*     Apply permutations IPIV to RHS
*
      CALL LA_LASWP( 1, RHS(1), MAX( 1, N ), 1, N-1, IPIV(1), 1 )
*
*     Solve for L part
*
      DO 20 I = 1, N - 1
         DO 10 J = I + 1, N
            RHS( J ) = RHS( J ) - A( J, I )*RHS( I )
   10    CONTINUE
   20 CONTINUE
*
*     Solve for U part
*
      SCALE = ONE
*
*     Check for scaling
*
      I = LA_IAMAX( N, RHS, 1 )
      IF( TWO*SMLNUM*ABS( RHS( I ) ).GT.ABS( A( N, N ) ) ) THEN
         TEMP = HALF / ABS( RHS( I ) )
         CALL LA_SCAL( N, TEMP, RHS( 1 ), 1 )
         SCALE = SCALE*TEMP
      END IF
*
      DO 40 I = N, 1, -1
         CTEMP = CONE / A( I, I )
         RHS( I ) = RHS( I )*CTEMP
         DO 30 J = I + 1, N
            RHS( I ) = RHS( I ) - RHS( J )*( A( I, J )*CTEMP )
   30    CONTINUE
   40 CONTINUE
*
*     Apply permutations JPIV to the solution (RHS)
*
      CALL LA_LASWP( 1, RHS(1), MAX( 1, N ), 1, N-1, JPIV(1), -1 )
      RETURN
*
*     End of CGESC2
*
      END
